#!/bin/bash -x

PATH=$PATH:/bin:/usr/bin:/sbin

echo "PATH = " $PATH

GREP=/bin/grep
PING=/bin/ping

if ! /usr/bin/id | $GREP -q root; then
  echo "Must be run as root"
  exit 1
fi

#echo "Checking for connectivity"
#if $PING -c1 www.google.com > /dev/null; then
#    echo "It worked"
#else
#    echo "No dice"
#    exit 2
#fi

# Programs used
IFCONFIG=/sbin/ifconfig
ROUTE=/sbin/route
IPTABLES=/usr/sbin/iptables
AWK=/usr/bin/awk

# which interface is which
WANIFACE=rmnet_data0
LANIFACE=ecm0

# just printing stuff out so I know what's what while troubleshooting.
echo $IFCONFIG
echo $ROUTE
echo $IPTABLES
echo $WANIFACE
echo $LANIFACE


setupWAN() {
  # try to populate the WANIP
  WANIP=$(ifconfig $WANIFACE | sed -n '2 p' | $AWK '{print $2}' | sed -e s/addr://)
  echo wanip: $WANIP
  WANMAC=$(ifconfig $WANIFACE | sed -n '1 p' | $AWK '{print $5}')
  echo wanmac: $WANMAC

  # Need to look up the management port in the configuration settings. For now, set to 8080
  MGMTPORT=8080
  echo mgmtport: $MGMTPORT

  # Need to look up the management IP in the configuration settings.
  # For now 192.168.2.2 so the legato build stuff still works
  MANAGEIP="192.168.2.2"
  echo manageip: $MANAGEIP
  MANAGEBASE=$(echo $MANAGEIP |  cut -d '.' -f 1-3)
  echo managebase: $MANAGEBASE
  MANAGEBROADCAST=$MANAGEBASE.255
  echo managembroadcast: $MANAGEBROADCAST

  # Get the network config required to move forward
  WANNETMASK=$(ifconfig $WANIFACE | sed -n '2 p' | $AWK '{print $3}' | sed -e s/Mask://)
  echo wannetmask: $WANNETMASK
  WANGATEWAY=$(ip route | $GREP "default" | $AWK '{print $3}')
  echo wangateway: $WANGATEWAY
  echo "Exiting setup WAN."
}

setupLAN() {
  # Set up the LAN interface variables
  LANSUBBASE=$(echo $WANIP |  cut -d '.' -f 1-3)
  echo lansubbase: $LANSUBBASE
  LANSUBNET=$LANSUBBASE.0
  echo lansubnet: $LANSUBNET
  LANNETMASK=255.255.255.0  # don't change this or router might not work
  LANBROADCAST=$LANSUBBASE.255
  echo lanbroadcast: $LANBROADCAST

  # Make sure we don't have a collision with the default route
  # If so, just choose a different LANIP
  if [[ "$(echo ${WANIP} | tr "." " " | $AWK '{ print $4 }')" == 254 ]] ; then
    LANIP=$LANSUBBASE.1
  else
    LANIP=$LANSUBBASE.254
  fi
  echo lanip: $LANIP
}

rewriteNetworkInterfaces() {
  # Now set up the interfaces script in case the wireless disconnects and we ifdown/ifup eth1
  cat > /etc/network/interfaces <<END
# interfaces(5) Used by ifup(8) and ifdown(8)

auto lo
iface lo inet loopback

auto $MANAGEIFACE
iface $MANAGEIFACE inet static
  address $MANAGEIP
  netmask 255.255.255.0
  broadcast $MANAGEBROADCAST

auto $LANIFACE
iface $LANIFACE inet static
  address $LANIP
  netmask $LANNETMASK
  broadcast $LANBROADCAST

#auto $WANIFACE #no auto per Danny
iface $WANIFACE inet manual
up ifconfig $WANIFACE up
up route add -host $WANGATEWAY dev $WANIFACE
up route add default gw $WANGATEWAY
# up /usr/bin/dhtest -m $WANMAC -i $WANIFACE
down route del $WANGATEWAY
# down route del -host $WANGATEWAY dev $WANIFACE

# Wireless interfaces
# iface wlan0 inet manual
#  pre-up /etc/init.d/tiwifi start
#  post-down /etc/init.d/tiwifi stop

END
}

flushIPtables() {
  echo "Flushing the IP tables"
  # Flush all IPtables rules
  $IPTABLES -F
  $IPTABLES -X
  $IPTABLES -t nat -F
  $IPTABLES -t nat -X
  $IPTABLES -t mangle -F
  $IPTABLES -t mangle -X
  $IPTABLES -P INPUT ACCEPT
  $IPTABLES -P FORWARD ACCEPT
  $IPTABLES -P OUTPUT ACCEPT
}

setupIfconfig() {
  echo "Setting up ifconfig to clear the current configuration, the pseudo gateway and management interface."

  # Clear the IP configuration from the WAN, we can't have an interface IP there anymore
  $IFCONFIG $WANIFACE 0.0.0.0 up

  # Bring up the LAN interface with the pseudo gateway
  $IFCONFIG $LANIFACE $LANIP netmask $LANNETMASK broadcast $LANBROADCAST

  # Bring up the management interface
  $IFCONFIG $MANAGEIFACE $MANAGEIP netmask $LANNETMASK up
}

setupRoute() {
  echo "Setting up the route."
  # Make sure the default route is pointing out the correct interface
  #echo "add -host" $WANGATEWAY $WANIFACE
  $ROUTE add -host $WANGATEWAY dev $WANIFACE
  $ROUTE add -host $MANAGEIP dev $MANAGEIFACE
  $ROUTE add default gw $WANGATEWAY

  # Now, make sure the kernel knows how to get to that IP
  $ROUTE add -host $WANIP dev $LANIFACE
}



setupIPtables() {
  # This is the magic that makes it all happen.
  #   Change anything from the LAN to look like it is coming from the
  #   WAN IP (which is what the cell company is expecting). That way,
  #   everything will return back to the thing on the inside.
  echo "$IPTABLES -t nat -I POSTROUTING -s $LANIP/$LANNETMASK -o $WANIFACE -j SNAT --to-source $WANIP"
  $IPTABLES -t nat -I POSTROUTING -s $LANIP/$LANNETMASK -o $WANIFACE -j SNAT --to-source $WANIP
  echo "$IPTABLES -t nat -I POSTROUTING -s $MANAGEIP -o $WANIFACE -j SNAT --to-source $WANIP"
  $IPTABLES -t nat -I POSTROUTING -s $MANAGEIP -o $WANIFACE -j SNAT --to-source $WANIP
  echo "$IPTABLES -t nat -I POSTROUTING -o $WANIFACE -j SNAT --to-source $WANIP"
  $IPTABLES -t nat -I POSTROUTING -o $WANIFACE -j SNAT --to-source $WANIP
  echo "$IPTABLES -A FORWARD -p tcp --tcp-flags SYN,RST SYN -j TCPMSS  --clamp-mss-to-pmtu"
  $IPTABLES -A FORWARD -p tcp --tcp-flags SYN,RST SYN -j TCPMSS  --clamp-mss-to-pmtu
  echo "$IPTABLES -t nat -A PREROUTING -p tcp -i $WANIFACE -d $WANIP --dport $MGMTPORT -j DNAT --to-destination $MANAGEIP:$MGMTPORT"
  $IPTABLES -t nat -A PREROUTING -p tcp -i $WANIFACE -d $WANIP --dport $MGMTPORT -j DNAT --to-destination $MANAGEIP:$MGMTPORT
  #echo "$IPTABLES -t nat -A PREROUTING -p tcp -i $WANIFACE -d $WANIP --dport 9012 -j DNAT --to-destination $MANAGEIP:9012"
  #$IPTABLES -t nat -A PREROUTING -p tcp -i $WANIFACE -d $WANIP --dport 9012 -j DNAT --to-destination $MANAGEIP:9012
  echo "$IPTABLES -A FORWARD -m state --state INVALID -j DROP"
  $IPTABLES -A FORWARD -m state --state INVALID -j DROP
  $IPTABLES -A INPUT -m state --state INVALID -j DROP
  $IPTABLES -A OUTPUT -m state --state INVALID -j DROP
  # After a kernel recompile, add this back in!
  #$IPTABLES -A POSTROUTING -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu


}

rewriteDnsmasqConf() {
  confFile="/etc/dnsmasq.conf"
  echo "" > $confFile
  echo "#/etc/dnsmasq.conf" >> $confFile
  # domain-needed blocks incomplete requests from leaving your network
  echo "domain-needed" >> $confFile
  # bogus-priv prevents non-routable private addresses from being forwarded out of your network
  echo "bogus-priv" >> $confFile
  echo "" >> $confFile
  
  echo "#Specify a domain name if you want. Or don't. Whatever. I don't care." >> $confFile
  echo "# But this is a sweet domain name. Just sayin' " >> $confFile
  echo "dhcp-option=15,myskyus.inseego.com" >> $confFile
  echo "" >> $confFile
  
  echo "expand-hosts" >> $confFile
  echo "local=/myskyus.inseego.com/ " >> $confFile
  echo "" >> $confFile
  echo "listen-address=127.0.0.1 " >> $confFile

  echo "bind-interfaces" >> $confFile
  echo "" >> $confFile
  echo "#Interfaces you want to listen on" >> $confFile
  echo "# If you don't know what this is, see someone about finding a new job" >> $confFile
  echo "interface=ecm0" >> $confFile
  echo "" >> $confFile
  
  echo "#One of these for each interface you want addresses on. This provides a single IP on ecm0" >> $confFile
  echo "# If you want multiple addresses the second should be the higher of the range." >> $confFile
  echo "# dhcp-range=ecm0,192.168.2.3,192.168.2.3,255.255.255.0,4h" >> $confFile
  echo "# Here is an excample of providing a range of addresses" >> $confFile
  echo "# dhcp-range=ecm0,192.168.2.100,192.168.2.200,255.255.255.0,4h" >> $confFile
  echo "dhcp-range=ecm0,$WANIP,$WANIP,255.255.255.0,4h" >> $confFile
  echo "" >> $confFile
  
  echo "#This is the gateway. You need one of these per interface if you are serving DHCP" >> $confFile
  echo "# Do not include this option if you don't want it to specify a router for clients." >> $confFile
  echo "# Not specifying a gateway will still assign IP addresses, just not a default route." >> $confFile
  echo "# A configuration without a gateway is useful for management only addresses." >> $confFile
  echo "#Also, this must be done interface specific and not globally if you are listening on more" >> $confFile
  echo "# than one interface." >> $confFile
  echo "#dhcp-option=ecm0,3,192.168.2.2" >> $confFile
  echo "dhcp-option=ecm0,3,$WANGATEWAY" >> $confFile
  echo "" >> $confFile
  
  echo "#Tell clients NOT to be routers. Best practice (ipforwarding off)" >> $confFile
  echo "# This means YOU are the router and NOT the client" >> $confFile
  echo "dhcp-option=19,0" >> $confFile
  
  
#  echo "dhcp-option=lan,6,???????" >> $confFile  # management address??? The second stanza tells our LAN clients to get their 
#  DNS from the Dnsmasq server.
  
  echo "#Specify up to 5 DNS servers" >> $confFile
  echo "# Just separate them by a comma and you'll be fine" >> $confFile
  echo "dhcp-option=6,8.8.8.8,8.8.8.4,208.67.220.220" >> $confFile


  echo "" >> $confFile   
  echo "" >> $confFile
  echo "" >> $confFile 


}

dnsmasqCleanupAndRestart() {
  # ToDo: Research and figure out if any clean-up is needed such as clearing the dhcp leases.
  
  # restart dnsmasq 
  /usr/bin/dnsmasq -7 /etc/dnsmasq.d
  
}

dnsmasqMagic() {
  # First, make sure it stopped in case it started by accident
  killall dnsmasq
  sleep 2
  rewriteDnsmasqConf;
  dnsmasqCleanupAndRestart;
}

main () {
  # Look at the configuration to see if ippass is on. For now, it's hard coded to on
  IPPASS="on"
  echo $IPPASS

  if [[ "x$IPPASS" = "xon" ]] 
    then

    echo "In the if/then statement"
    echo "setupWAN"
    setupWAN;
    echo "setupLAN"
    setupLAN;

    # Make subinterface for management
    echo "Make subinterface"
    MANAGEIFACE=$LANIFACE:0
    echo manageiface: $MANAGEIFACE
    flushIPtables;
    killall dnsmasq;
    echo "setupIfconfig"
    setupIfconfig;
    echo "setupRoute"
    setupRoute;

    # enable proxy_arp so to make the magic happen
    echo "Enable proxy_arp"
    echo 1 >/proc/sys/net/ipv4/conf/$WANIFACE/proxy_arp
    echo 1 >/proc/sys/net/ipv4/conf/$LANIFACE/proxy_arp
    echo "setupIPtables"
    setupIPtables;

    # Turn it on!!!!
    echo "do ip_forward"
    echo 1 >/proc/sys/net/ipv4/ip_forward
    echo "dnsmasqMagic"
    dnsmasqMagic;
    echo "rewriteNetworkInterfaces"
    rewriteNetworkInterfaces;
  fi


}

main
exit 0
